@@ -2,7 +2,7 @@
 # Apache License 2.0, Copyright 2019 Tebel.Automation Private Limited
 # https://github.com/tebelorg/RPA-Python/blob/master/LICENSE.txt
 __author__ = 'Ken Soh <opensource@tebel.org>'
-__version__ = '1.39.0'
+__version__ = '1.42.0'
 
 import subprocess
 import os
@@ -37,6 +37,9 @@ _tagui_id = 0
 # to track the original directory when init() was called
 _tagui_init_directory = ''
 
+# to track file download directory for web browser
+_tagui_download_directory = ''
+
 # to track location of TagUI (default user home folder)
 if platform.system() == 'Windows':
     _tagui_location = os.environ['APPDATA']
@@ -429,7 +432,7 @@ def setup():
 def init(visual_automation = False, chrome_browser = True, headless_mode = False):
     """start and connect to tagui process by checking tagui live mode readiness"""
 
-    global _process, _tagui_started, _tagui_id, _tagui_visual, _tagui_chrome, _tagui_init_directory
+    global _process, _tagui_started, _tagui_id, _tagui_visual, _tagui_chrome, _tagui_init_directory, _tagui_download_directory
 
     if _tagui_started:
         print('[RPA][ERROR] - use close() before using init() again')
@@ -564,7 +567,10 @@ def init(visual_automation = False, chrome_browser = True, headless_mode = False
                 _tagui_id = _tagui_id + 1
 
                 # set variable to track original directory when init() was called
-                _tagui_init_directory = os.getcwd() 
+                _tagui_init_directory = os.getcwd()
+
+                # set variable to track file download directory for web browser 
+                _tagui_download_directory = os.getcwd()
 
                 return True
 
@@ -1224,6 +1230,33 @@ def ask(text_to_prompt = ''):
         else:
             return input(text_to_prompt + space_padding)
 
+def telegram(telegram_id = None, text_to_send = None):
+    if telegram_id is None or telegram_id == '':
+        print('[RPA][ERROR] - Telegram ID missing for telegram()')
+        return False
+
+    if text_to_send is None or text_to_send == '':
+        print('[RPA][ERROR] - text message missing for telegram()')
+        return False
+
+    # in case number is given instead of string
+    telegram_id = str(telegram_id)
+
+    telegram_endpoint = 'https://tebel.org/rpapybot'
+    telegram_params = {'chat_id': telegram_id, 'text': text_to_send}
+
+    if _python2_env():
+        import json; import urllib
+        telegram_endpoint = telegram_endpoint + '/sendMessage.php?' + urllib.urlencode(telegram_params)
+        telegram_response = urllib.urlopen(telegram_endpoint).read()
+        return json.loads(telegram_response)['ok']
+        
+    else:
+        import json; import urllib.request; import urllib.parse
+        telegram_endpoint = telegram_endpoint + '/sendMessage.php?' + urllib.parse.urlencode(telegram_params)
+        telegram_response = urllib.request.urlopen(telegram_endpoint).read()
+        return json.loads(telegram_response)['ok']
+
 def keyboard(keys_and_modifiers = None):
     if not _started():
         print('[RPA][ERROR] - use init() before using keyboard()')
@@ -1279,7 +1312,9 @@ def table(element_identifier = None, filename_to_save = None):
         print('[RPA][ERROR] - filename missing for table()')
         return False
 
-    elif not exist(element_identifier):
+    element_identifier = str(element_identifier)
+
+    if not exist(element_identifier):
         print('[RPA][ERROR] - cannot find ' + element_identifier)
         return False
 
@@ -1668,3 +1703,32 @@ def clipboard(text_to_put = None):
 
     else:
         return True
+
+def download_location(location = None):
+    global _tagui_download_directory
+    if not _started():
+        print('[RPA][ERROR] - use init() before using download_location()')
+        return False
+
+    if location is None:
+        return _tagui_download_directory
+
+    if "'" in location:
+        print('[RPA][ERROR] - single quote in location not supported here')
+        return False
+
+    if platform.system() == 'Windows':
+        location = location.replace('/', '\\')
+
+    if not send("chrome_step('Page.setDownloadBehavior',{behavior: 'allow', downloadPath: '" + location + "'});"):
+        return False
+
+    else:
+        _tagui_download_directory = location
+        return True
+
+def get_text(text = None, left = None, right = None, count = 1):
+    if text is None or left is None or right is None:
+        return ''
+
+    return text
